--    Copyright (C) 2004 by neuron                                          
--    aagaande@gmail.com                                             
--                                                                          
--    This program is free software; you can redistribute it and--or modify  
--    it under the terms of the GNU General Public License as published by  
--    the Free Software Foundation; either version 2 of the License, or     
--    (at your option) any later version.                                   
--                                                                          
--    This program is distributed in the hope that it will be useful,       
--    but WITHOUT ANY WARRANTY; without even the implied warranty of        
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         
--    GNU General Public License for more details.                          
--                                                                          
--    You should have received a copy of the GNU General Public License     
--    along with this program; if not, write to the                         
--    Free Software Foundation, Inc.,                                       
--    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 

local dewdrop = AceLibrary("Dewdrop-2.0")
local quixote = AceLibrary("Quixote-1.0")
local crayon = AceLibrary("Crayon-2.0")
local tablet = AceLibrary("Tablet-2.0")

local L = AceLibrary("AceLocale-2.2"):new("nQuestLog-Events")
L:RegisterTranslations("enUS", function() return {
	["Visible"] = true,
	["Hidden"] = true,
	["Hidden (no quests to track)"] = true,
	["Tracker "] = true,

	["\n|cffeda55fClick|r to toggle Quest Tracker\n|cffeda55fRight Click|r for options"] = true,

	["Zone visibility:"] = true,
	["Toggle showing "] = true,
	["Toggle zone visibility"] = true,

	["Hide Zone"] = true,
	["Show Zone"] = true,
	
	["Lock minion"] = true,
} end)    

L:RegisterTranslations("koKR", function() return {
	["Visible"] = "보임",
	["Hidden"] = "숨김",
	["Hidden (no quests to track)"] = "숨김 (퀘스트 없음)",
	["Tracker "] = "알림창 ",

	["\n|cffeda55fClick|r to toggle Quest Tracker\n|cffeda55fRight Click|r for options"] = "\n|cffeda55f클릭|r하면 퀘스트 알림창을 표시합니다.\n|cffeda55f오른쪽 클릭|r하면 설정창을 엽니다.",

	["Zone visibility:"] = "지역 보기",
	["Toggle showing "] = "보기 토글",
	["Toggle zone visibility"] = "지역 보기 토글",

	["Hide Zone"] = "지역 숨기기",
	["Show Zone"] = "지역 보기",
	
	["Lock minion"] = "알림창 고정",
} end)    

L:RegisterTranslations("deDE", function() return {
	["Visible"] = "Sichtbar",
	["Hidden"] = "Versteckt",
	["Hidden (no quests to track)"] = "Versteckt (keine Quests zum Verfolgen)",
	["Tracker "] = "Tracker ",

	["\n|cffeda55fClick|r to toggle Quest Tracker\n|cffeda55fRight Click|r for options"] = "\n|cffeda55fKlicken|r um den Tracker ein-/auszublenden\n|cffeda55fRechtsklick|r f\195\188r die Optionen",

	["Zone visibility:"] = "Gebietssichtbarkeit",
	["Toggle showing "] = "Anzeigen ein/aus",
	["Toggle zone visibility"] = "Gebietssichtbarkeit ein/aus",

	["Hide Zone"] = "Gebiet verstecken",
	["Show Zone"] = "Gebiet anzeigen",
	
	["Lock minion"] = "Tracker festsetzen",
} end)

-------------------------------------------------- UI Events : --------------------------------------------------

function nQuestLog:OnTooltipUpdate()
	local cat = tablet:AddCategory()
	
	local strStatus = L["Visible"]
	if (not nQuestLogFrame:IsVisible()) then
		strStatus = L["Hidden"]
		if (self.db.char["visible"]) then
			strStatus = L["Hidden (no quests to track)"]
		end
	end
	
	cat:AddLine("text",L["Tracker "] .. strStatus)
	
	tablet:SetHint(L["\n|cffeda55fClick|r to toggle Quest Tracker\n|cffeda55fRight Click|r for options"])
end

function nQuestLog:OnClick(button)
	-- FuBar OnClick
	if (nQuestLogFrame:IsVisible()) then
		self:ToggleVisible(false)
	else
		self:ToggleVisible(true)
	end
end

function nQuestLog:OnTextUpdate()
	local iQuests = quixote:GetNumQuests()
	local color = crayon:GetThresholdHexColor((MAX_QUESTS - iQuests) / MAX_QUESTS)
	self:SetText(crayon:Colorize(color, iQuests))
end

function nQuestLog:OnMenuRequest()
	self:RefreshMenu()
	dewdrop:FeedAceOptionsTable(self.mainMenu)
end

function nQuestLog:OnQuestLogClick(button,questid,zone)
	-- if questid is nil then it's a zone
	if (IsShiftKeyDown()) then
		if (questid) then
			nQuestLog:OnQuestLogShiftClick(button,questid)
		end
		return
	end
	
	if (button == "RightButton" and IsAltKeyDown()) then
		self:OnQuestLogAltRightClick()
	elseif (questid == nil or button == "RightButton") then
		-- We do if button == "RightButton" to open the zone menu on rightclick
		self:OnQuestLogClickZone(button,this.strZone)
	else
		self:OnQuestLogClickQuest(button,questid,zone)
	end
end

function nQuestLog:OnQuestLogAltRightClick()
	-- Todo, rename these functions to something more sane, and maybe move em to another file.
	local popupMenu = {
		type='group',
		args = {
			header = {
				type="header",
				name=L["Zone visibility:"],
				order=1,
			}
		},
	}
	
	for i,zone in quixote:IterZones() do
		prettyzone = self:Colorize(zone,"zoneHeader")
		popupMenu.args[zone] = {
			order = i + 5,
			name = prettyzone,
			type = "toggle",
			desc = L["Toggle showing "] .. zone,
			get = function()
				return not self.db.char.hiddenZones[zone]
			end,
			set = function(v)
				self.db.char.hiddenZones[zone] = nil
				if (not v) then
					self.db.char.hiddenZones[zone] = true
				end
				self:Refresh()
			end
		}
	end
	
	dewdrop:Open( nQuestLogFrame,
		'children', function() dewdrop:FeedAceOptionsTable(popupMenu ) end,
		'cursorX', true, 'cursorY', true )
end

function nQuestLog:OnQuestLogShiftClick(button,questid)
	local title,level,tag,_,_,objectives = quixote:GetQuestById(questid)
	if (title == nil) then
		return
	end
	
	-- Link quest to chat
	local text = "[" .. level .. tag .. "] " .. title .. " "
	local totalGot,totalNeeded = 0,0
	
	
	local extraText = ""
	
	if (objectives > 0) then
		for i=1,objectives do
			local desc, _, got, needed = quixote:GetQuestStatusById(questid, i)
			totalGot = totalGot + got
			totalNeeded = totalNeeded + needed
			if (button == "RightButton") then
				-- Link quest objectives
				if (i > 1) then
					extraText = extraText .. ", "
				end
				extraText = extraText .. desc .. " " .. got .. "/" .. needed
			end
		end
		
		if (extraText ~= "") then
			text = text .. "(" .. extraText .. ") "
		end
	end
	
	if (IsControlKeyDown()) then
		-- Link total objectives
		text = text .. totalGot .. "/" .. totalNeeded .. " "
	end
			
	if (ChatFrameEditBox:IsVisible()) then
		ChatFrameEditBox:Insert(text)
	else
		local chatType = DEFAULT_CHAT_FRAME.editBox:GetAttribute("chatType")
		SendChatMessage(text, chatType)
	end
end

function nQuestLog:OnQuestLogClickZone(button,zone)
	if (button == "LeftButton") then
		-- Fold/Unfold zone
		if self.db.char.hiddenZones[zone] then
			self.db.char.hiddenZones[zone] = nil
		else
			self.db.char.hiddenZones[zone] = true
		end
		self:Refresh()
	elseif (button == "RightButton" and not self.db.profile.trackerLocked) then
		-- Tracker unlocked on right mouse button, lets show a lock menu
		local popupMenu = {
			type='group',
			args = {
		         showHide = {
		            type = 'execute',
		            name = L["Lock minion"],
		            desc = L["Lock minion"],
		            order = 1,
		            func = function()
		            	self.db.profile.trackerLocked = true
		            	dewdrop:Close()
		            end
		        },
			}
		}
		dewdrop:Open( nQuestLogFrame,
			'children', function() dewdrop:FeedAceOptionsTable(popupMenu ) end,
			'cursorX', true, 'cursorY', true )
			
	elseif (button == "RightButton") then
		-- Show popup menu for this zone
		local zoneVisibility = L["Hide Zone"]
		if self.db.char.hiddenZones[zone] then
			zoneVisibility = L["Show Zone"]
		end
		zoneVisibility = self:Colorize(zoneVisibility,"zoneHeader")
		
		local popupMenu = {
			type='group',
			args = {
		         showHide = {
		            type = 'execute',
		            name = zoneVisibility,
		            desc = L["Toggle zone visibility"],
		            order = 1,
		            func = function()
		            	if (self.db.char.hiddenZones[zone]) then
		            		self.db.char.hiddenZones[zone] = nil
		            	else
		            		self.db.char.hiddenZones[zone] = true
		            	end
		            	self:Refresh()
		            	dewdrop:Close()
		            end
		        },
			}
		}
		
		for _,id in quixote:QuestsByZone(zone) do
			local title, level, tag, iGroup, complete, iObjectives = quixote:GetQuestById(id)
			local objective = quixote:GetQuestText(id)
			local _, got, needed = self:Format_QuestObjectives(title,iObjectives,objective,id)
			local formatedTitle = self:Format_QuestTitle(title,level,tag,got,needed,complete)
			
			popupMenu.args[title] = {
				order = id + 5,
				name = formatedTitle,
				type = "toggle",
				desc = L["Toggle showing "] .. title,
				get = function()
					return self.db.char.watchedQuests[title]
				end,
				set = function(v)
					self.db.char.watchedQuests[title] = nil
					if (v) then
						self.db.char.watchedQuests[title] = true
					end
					self:Refresh()
				end
			}
		end
		
		dewdrop:Open( nQuestLogFrame,
			'children', function() dewdrop:FeedAceOptionsTable(popupMenu ) end,
			'cursorX', true, 'cursorY', true )
	end
end

function nQuestLog:ShowLog(questid)
	-- Show details frame
	if self:IsModuleEnabled("DetailsFrame") then
		self:GetModule("DetailsFrame"):ShowLog(questid)
	else
		local wasHidden = not QuestLogFrame:IsVisible()
		HideUIPanel(QuestLogFrame)
		
		if (GetQuestLogSelection() ~= questid or wasHidden) then
			SelectQuestLogEntry(questid)
			QuestLog_SetSelection(questid)
			ShowUIPanel(QuestLogFrame)
			if LightHeaded then
				LightHeaded:QuestLogTitleButton_OnClick()
			end
		end
	end
end

function nQuestLog:OnQuestLogClickQuest(button,questid,zone)
	local title,level,_,_,_,objectives = quixote:GetQuestById(questid)
	if (title == nil) then
		return
	end


	if (button == "LeftButton") then
		if (IsAltKeyDown()) then
			-- Toggle objective visibility
			if self:ShouldShowObjective(title) then
				self.db.char.watchedObjectives[title] = false
			else
				self.db.char.watchedObjectives[title] = true
			end
			self:Refresh()
		else
			self:ShowLog(questid)
		end
	end
end

-------------------------------------------------- Quest Events: --------------------------------------------------

function nQuestLog:OnQuestLog_Ready()
	local newDb = true
	
	-- Lets clean the databases as well:
	local function CleanUp(database)
		for name in pairs(database) do
			if not quixote:GetQuestByName(name) then
				database[name] = nil
			end
			newDb = false
		end
	end
	
	local function CleanUpZones()
		for name in pairs(self.db.char.hiddenZones) do
			local _,zones = quixote:QuestsByZone(name)
			if (not zones) then
				self.db.char.hiddenZones[name] = nil
			end
			newDb = false
		end
	end
	
	CleanUp(self.db.char.watchedQuests)
	CleanUp(self.db.char.watchedObjectives)
	CleanUpZones()
	
	if (newDb) then
		-- This has got to be a new user, lets add all his quests
		for _,id in quixote:QuestsByLevel() do
			local title = quixote:GetQuestById(id)
			self.db.char.watchedQuests[title] = true
		end
		
		if self:IsModuleEnabled("SmartVisibility") then
			self:GetModule("SmartVisibility"):OnZoneChange()
		end
		if self:IsModuleEnabled("Highlight") then
			self:GetModule("Highlight"):OnZoneChange()
		end
	end
	
	--Refresh and show db.
	self.refreshHold = nil
	if self.db.char["visible"] then
		self:ToggleVisible(true)
	end
	self:UpdateText()
end

function nQuestLog:OnQuest_Gained(strTitle)
	if (self.db.profile.trackNewQuests) then
		self.db.char.watchedQuests[strTitle] = true
	end
	self:UpdateText()
end

function nQuestLog:OnQuest_Lost(strTitle)
	self.db.char.watchedQuests[strTitle] = nil
	self.db.char.watchedObjectives[strTitle] = nil
	self:UpdateText()
end

function nQuestLog:OnQuest_Leaderboard_Update(title, _, _, desc)
	if (self.db.profile.trackWhenLooting and not self.db.char.watchedQuests[title]) then
		self.db.char.watchedQuests[title] = true
		self:Print("Found " .. desc .. " for " .. title .. " adding to quest tracker.")
	end
end

-------------------------------------------------- Refresh checker --------------------------------------------------

function nQuestLog:OnRefreshCheck()
	-- Modules that needs refresh and that triggers any zone/quest event should not do a manual update, but set this flag instead.
	if self.refresh then
		self:Refresh()
	end
end
